@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -26,6 +26,11 @@
 
 #ifndef _BOOL_H
 #define _BOOL_H
+
+#if defined HAVE_CONFIG_H && !defined VERSION
+#include "config.h"
+#endif
+
 /* proposed by info autoconf */
 #ifdef HAVE_STDBOOL_H
 #include <stdbool.h>
