@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1985 Supoj Sutanthavibul
  * Copyright (c) 1991 Micah Beck
- * Parts Copyright (c) 1989-2010 by Brian V. Smith
- * Parts Copyright (c) 2015,2016 Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -46,7 +46,7 @@
 #include "pi.h"
 
 #include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "object.h"
 #include "bound.h"
 #include "localmath.h"
 
@@ -82,51 +82,53 @@ struct _arrow_shape arrow_shapes[NUMARROWS] = {	/* NUMARROWS def'd in fig2dev.h
 	/* number of points, index of tip, {datapairs} */
 	/* last point must be upper-left point of tail */
 	/* type 0 */
-	{ 3, 1, 0, 0, true, false, 2.15, {{-1,-0.5}, {0,0}, {-1,0.5}}},
+	{ 3, 1, 0, 0, true, false, 2.15, {{-1,-0.5}, {0,0}, {-1,0.5}}, {{0,0}}},
 	/* place holder for what would be type 0 filled */
-	{ 0 },
+	/* Make this a copy of the default arrow above. Maliciously crafted
+	   fig files might choose this arrow, hence provide a valid shape. */
+	{ 3, 1, 0, 0, true, false, 2.15, {{-1,-0.5}, {0,0}, {-1,0.5}}, {{0,0}}},
 	/* type 1a simple triangle */
 	{ 4, 1, 0, 2, true, false, 2.1,
-		{{-1.0,0.5}, {0,0}, {-1.0,-0.5}, {-1.0,0.5}}},
+		{{-1.0,0.5}, {0,0}, {-1.0,-0.5}, {-1.0,0.5}}, {{0,0}}},
 	/* type 1b filled simple triangle*/
 	{ 4, 1, 0, 2, true, false, 2.1,
-		{{-1.0,0.5}, {0,0}, {-1.0,-0.5}, {-1.0,0.5}}},
+		{{-1.0,0.5}, {0,0}, {-1.0,-0.5}, {-1.0,0.5}}, {{0,0}}},
 	/* type 2a concave spearhead */
 	{ 5, 1, 0, 2, true, false, 2.6,
-		{{-1.25,0.5},{0,0},{-1.25,-0.5},{-1.0,0},{-1.25,0.5}}},
+		{{-1.25,0.5},{0,0},{-1.25,-0.5},{-1.0,0},{-1.25,0.5}}, {{0,0}}},
 	/* type 2b filled concave spearhead */
 	{ 5, 1, 0, 2, true, false, 2.6,
-		{{-1.25,0.5},{0,0},{-1.25,-0.5},{-1.0,0},{-1.25,0.5}}},
+		{{-1.25,0.5},{0,0},{-1.25,-0.5},{-1.0,0},{-1.25,0.5}}, {{0,0}}},
 	/* type 3a convex spearhead */
 	{ 5, 1, 0, 2, true, false, 1.5,
-		{{-0.75,0.5},{0,0},{-0.75,-0.5},{-1.0,0},{-0.75,0.5}}},
+		{{-0.75,0.5},{0,0},{-0.75,-0.5},{-1.0,0},{-0.75,0.5}}, {{0,0}}},
 	/* type 3b filled convex spearhead */
 	{ 5, 1, 0, 2, true, false, 1.5,
-		{{-0.75,0.5},{0,0},{-0.75,-0.5},{-1.0,0},{-0.75,0.5}}},
+		{{-0.75,0.5},{0,0},{-0.75,-0.5},{-1.0,0},{-0.75,0.5}}, {{0,0}}},
 	/* type 4a diamond */
 	{ 5, 1, 0, 2, true, false, 1.15,
-		{{-0.5,0.5},{0,0},{-0.5,-0.5},{-1.0,0},{-0.5,0.5}}},
+		{{-0.5,0.5},{0,0},{-0.5,-0.5},{-1.0,0},{-0.5,0.5}}, {{0,0}}},
 	/* type 4b filled diamond */
 	{ 5, 1, 0, 2, true, false, 1.15,
-		{{-0.5,0.5},{0,0},{-0.5,-0.5},{-1.0,0},{-0.5,0.5}}},
+		{{-0.5,0.5},{0,0},{-0.5,-0.5},{-1.0,0},{-0.5,0.5}}, {{0,0}}},
 	/* type 5a/b circle - handled in code */
-	{ 0, 0, 0, 0, true, false, 0.0 },
-	{ 0, 0, 0, 0, true, false, 0.0 },
+	{ 0, 0, 0, 0, true, false, 0.0, {{0,0}}, {{0,0}} },
+	{ 0, 0, 0, 0, true, false, 0.0, {{0,0}}, {{0,0}} },
 	/* type 6a/b half circle - handled in code */
-	{ 0, 0, 0, 0, true, false, -1.0 },
-	{ 0, 0, 0, 0, true, false, -1.0 },
+	{ 0, 0, 0, 0, true, false, -1.0, {{0,0}}, {{0,0}} },
+	{ 0, 0, 0, 0, true, false, -1.0, {{0,0}}, {{0,0}} },
 	/* type 7a square */
 	{ 5, 1, 0, 3, true, false, 0.0,
-		{{-1.0,0.5},{0,0.5},{0,-0.5},{-1.0,-0.5},{-1.0,0.5}}},
+		{{-1.0,0.5},{0,0.5},{0,-0.5},{-1.0,-0.5},{-1.0,0.5}}, {{0,0}}},
 	/* type 7b filled square */
 	{ 5, 1, 0, 3, true, false, 0.0,
-		{{-1.0,0.5},{0,0.5},{0,-0.5},{-1.0,-0.5},{-1.0,0.5}}},
+		{{-1.0,0.5},{0,0.5},{0,-0.5},{-1.0,-0.5},{-1.0,0.5}}, {{0,0}}},
 	/* type 8a reverse triangle */
 	{ 4, 1, 0, 1, true, false, 0.0,
-		{{0,0.5},{0,-0.5},{-1.0,0},{0,0.5}}},
+		{{0,0.5},{0,-0.5},{-1.0,0},{0,0.5}}, {{0,0}}},
 	/* type 8b filled reverse triangle */
 	{ 4, 1, 0, 1, true, false, 0.0,
-		{{0,0.5},{0,-0.5},{-1.0,0},{0,0.5}}},
+		{{0,0.5},{0,-0.5},{-1.0,0},{0,0.5}}, {{0,0}}},
 	/* type 9a top-half filled concave spearhead */
 	{ 5, 1, 3, 2, false, false, 2.6,
 		{{-1.25,0.5},{0,0},{-1.25,-0.5},{-1.0,0},{-1.25,0.5}},
@@ -137,32 +139,32 @@ struct _arrow_shape arrow_shapes[NUMARROWS] = {	/* NUMARROWS def'd in fig2dev.h
 		{{-1.25,0.5},{0,0},{-1,0}}},
 	/* type 10o top-half simple triangle */
 	{ 4, 1, 0, 2, true, true, 2.5,
-		{{-1.0,0.5}, {0,0}, {-1,0.0}, {-1.0,0.5}}},
+		{{-1.0,0.5}, {0,0}, {-1,0.0}, {-1.0,0.5}}, {{0,0}}},
 	/* type 10f top-half filled simple triangle*/
 	{ 4, 1, 0, 2, true, true, 2.5,
-		{{-1.0,0.5}, {0,0}, {-1,0.0}, {-1.0,0.5}}},
+		{{-1.0,0.5}, {0,0}, {-1,0.0}, {-1.0,0.5}}, {{0,0}}},
 	/* type 11o top-half concave spearhead */
 	{ 4, 1, 0, 2, true, true, 3.5,
-		{{-1.25,0.5}, {0,0}, {-1,0}, {-1.25,0.5}}},
+		{{-1.25,0.5}, {0,0}, {-1,0}, {-1.25,0.5}}, {{0,0}}},
 	/* type 11f top-half filled concave spearhead */
 	{ 4, 1, 0, 2, true, true, 3.5,
-		{{-1.25,0.5}, {0,0}, {-1,0}, {-1.25,0.5}}},
+		{{-1.25,0.5}, {0,0}, {-1,0}, {-1.25,0.5}}, {{0,0}}},
 	/* type 12o top-half convex spearhead */
 	{ 4, 1, 0, 2, true, true, 2.5,
-		{{-0.75,0.5}, {0,0}, {-1,0}, {-0.75,0.5}}},
+		{{-0.75,0.5}, {0,0}, {-1,0}, {-0.75,0.5}}, {{0,0}}},
 	/* type 12f top-half filled convex spearhead */
 	{ 4, 1, 0, 2, true, true, 2.5,
-		{{-0.75,0.5}, {0,0}, {-1,0}, {-0.75,0.5}}},
+		{{-0.75,0.5}, {0,0}, {-1,0}, {-0.75,0.5}}, {{0,0}}},
 	/* type 13a "wye" */
-	{ 3, 0, 0, 0, true, false, -1.0, {{0,-0.5},{-1.0,0},{0,0.5}}},
+	{ 3, 0, 0, 0, true, false, -1.0, {{0,-0.5},{-1.0,0},{0,0.5}}, {{0,0}}},
 	/* type 13b bar */
-	{ 2, 1, 0, 0, true, false, 0.0, {{0,-0.5},{0,0.5}}},
+	{ 2, 1, 0, 0, true, false, 0.0, {{0,-0.5},{0,0.5}}, {{0,0}}},
 	/* type 14a two-prong fork */
 	{ 4, 0, 0, 0, true, false, -1.0,
-		{{0,-0.5},{-1.0,-0.5},{-1.0,0.5},{0,0.5}}},
+		{{0,-0.5},{-1.0,-0.5},{-1.0,0.5},{0,0.5}}, {{0,0}}},
 	/* type 14b backward two-prong fork */
 	{ 4, 1, 0, 0, true, false, 0.0,
-		{{-1.0,-0.5,},{0,-0.5},{0,0.5},{-1.0,0.5}}}
+		{{-1.0,-0.5,},{0,-0.5},{0,0.5},{-1.0,0.5}}, {{0,0}}}
 };
 
 void
@@ -870,7 +872,7 @@ calc_arrow(int x1, int y1, int x2, int y2, int linethick, F_arrow *arrow,
 	lpt = 0.0;
 	thk = THICKNESS(arrow->thickness);
 	if (tipmv > 0.0)
-	    lpt = thk * sqrt(wd*wd + tipmv*tipmv*len*len) / 2. / wd;
+	    lpt = thk * sqrt(wd*wd + tipmv*tipmv*len*len) / 2. / fabs(wd);
 	else if (tipmv == 0.0)
 	    lpt = thk / 2.0;	/* types which have blunt end */
 			    /* (Don't adjust those with tipmv < 0) */
@@ -903,7 +905,7 @@ calc_arrow(int x1, int y1, int x2, int y2, int linethick, F_arrow *arrow,
 	    /*
 	     * CIRCLE and HALF-CIRCLE arrowheads
 	     *
-	     * We approximate circles with (40+zoom)/4 points
+	     * We approximate circles with 40 points
 	     */
 	    double	maxx;
 	    double	fix_x, fix_y, xs, ys;
@@ -934,8 +936,8 @@ calc_arrow(int x1, int y1, int x2, int y2, int linethick, F_arrow *arrow,
 	    dy = my - ys;
 	    fix_x = xs + (dx / 2.0);
 	    fix_y = ys + (dy / 2.0);
-	    /* choose number of points for circle - 40+mag/4 points */
-	    *npoints = np = round(mag/4.0) + 40;
+	    /* choose number of points for circle */
+	    *npoints = np = 40;
 
 	    if (type == 5) {
 		/* full circle */
@@ -1093,16 +1095,15 @@ compute_arcarrow_angle(double x1, double y1, double x2, double y2,
 	r=sqrt(dx*dx+dy*dy);
 	h = (double) arrow->ht;
 	/* lines are made a little thinner in set_linewidth */
-	thick = (arrow->thickness <= THICK_SCALE) ?
-		    0.5* arrow->thickness :
-		    arrow->thickness - THICK_SCALE;
+	thick = arrow->thickness <= THICK_SCALE ?
+		0.5 * arrow->thickness : arrow->thickness - THICK_SCALE;
 	/* lpt is the amount the arrowhead extends beyond the end of the line */
 	lpt = thick/2.0/(arrow->wid/h/2.0);
 	/* add this to the length */
 	h += lpt;
 
-	/* radius too small for this method, use normal method */
-	if (h > 2.0*r) {
+	/* secant would be too large or too small */
+	if (h > 2.0*r || h < 0.01*r) {
 	    arc_tangent_int(x1,y1,x2,y2,direction,x,y);
 	    return;
 	}
