@@ -1,8 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Parts Copyright (c) 1999 by Philippe Bekaert
- * Parts Copyright (c) 1999-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -15,6 +16,7 @@
  *
  */
 
+
 /*
  * gencgm.c: convert fig to clear text version-1 Computer Graphics Metafile
  *
@@ -64,20 +66,15 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdarg.h>
 #include <string.h>
 #include <math.h>
-#include <limits.h>
-#include "pi.h"
 
-#include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
+#include "messages.h"
+#include "pi.h"
 
 #define	UNDEFVALUE	-100	/* undefined attribute value */
 #define	FILL_COLOR_INDEX 999	/* special color index for solid filled shapes.
@@ -148,9 +145,11 @@ gencgm_start(F_compound *objects)
 {
   int	 i;
   char	*p, *figname;
+  char	*figname_buf = NULL;
 
   if (from) {
-	figname = strdup(from);
+	figname_buf = strdup(from);
+	figname = figname_buf;
 	p = strrchr(figname, '/');
 	if (p)
 	    figname = p+1;	/* remove path from name for comment in file */
@@ -252,6 +251,8 @@ gencgm_start(F_compound *objects)
     print_comments("% ",objects->comments, " %");
     fprintf(tfp,"%% %%\n");
   }
+  if (figname_buf)
+    free(figname_buf);
 }
 
 int
@@ -549,6 +550,8 @@ hatchindex(index)
 static void
 getrgb(int color, int *r, int *g, int *b)
 {
+  if (color < 0)	/* DEFAULT color is black */
+    color = 0;
   if (color < NUM_STD_COLS) {
     *r = stdcols[color].r * 255.;
     *g = stdcols[color].g * 255.;
