@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2021 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -57,10 +57,11 @@
 #include <string.h>
 #include <math.h>
 
-#include "fig2dev.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
 #include "alloc.h"
 #include "creationdate.h"
+#include "messages.h"
 
 #define GBX_DRIVER_VERSION "0.1.1"
 #define MAX_APERTURES 989
@@ -523,6 +524,17 @@ flash_xy(long int x, long int y, int aperture_index )
 }
 
 
+/** Abort if the maximum number of apertures is exceeded. */
+static void
+abort_too_many_apertures(long int ap_index)
+{
+	if (ap_index < MAX_APERTURE_INDEX)
+		return;
+	put_msg("Error: Maximum number of possible apertures (%d) exceeded.",
+			MAX_APERTURE_INDEX);
+	exit(EXIT_FAILURE);
+}
+
 /** Define a filled circular aperture given the diameter.
 
     Returns the corresponding aperture index.
@@ -551,6 +563,7 @@ ad_aperture_define_circ(double outer_dia)
 
 	/* If the aperture has not been defined, add it. */
 	if (ap_index == -1) {
+		abort_too_many_apertures(count_apertures);
 		ap_index = count_apertures;
 		fprintf(tfp, "%%ADD%liC,%f%%*\n",
 				d_select_aperture + ap_index, outer_dia );
@@ -639,6 +652,7 @@ ad_aperture_define_square(double width)
 
 	/* If the aperture has not been defined, add it. */
 	if (ap_index == -1) {
+		abort_too_many_apertures(count_apertures);
 		ap_index = count_apertures;
 		fprintf(tfp, "%%ADD%liR,%fX%f*%%\n",
 				d_select_aperture + ap_index, width, width);
@@ -817,9 +831,7 @@ gengbx_option(char opt, char *optarg)
 void
 gengbx_start(F_compound *objects)
 {
-	/* struct paperdef *pd; */
 	char    stime[CREATION_TIME_LEN];
-	/*     long int     i; */
 
 	gbx_scale_factor=pow(10,gbx_after);
 
@@ -1157,17 +1169,22 @@ gengbx_line (F_line *l)
 				draw_xy(p->x, p->y);
 				p = p->next;
 				draw_xy(p->x, p->y);
-				move_xy(p->x, p->y);
+				/* This is probably not correct for
+				   lines having 4 points. */
+				if (p->next) {
+					move_xy(p->x, p->y);
+					/* Draw the remainder - using
+					   the join style. */
+					for (; p->next->next; p = p->next)
+						draw_xy(p->x, p->y);
 
-				/* Draw the remainder - using the join style. */
-				for (; p->next->next; p = p->next)
+					/* Add end section of line. */
+					use_aperture(
+					       define_aperture_for_line_end(l));
+					move_xy(p->x, p->y);
+					p = p->next;
 					draw_xy(p->x, p->y);
-
-				/* Add end section of line. */
-				use_aperture(define_aperture_for_line_end(l));
-				move_xy(p->x, p->y);
-				p = p->next;
-				draw_xy(p->x, p->y);
+				}
 			}
 		}  else {
 			/* All other lines get drawn like this. */
@@ -1398,7 +1415,7 @@ gengbx_text (F_text *t)
 struct driver dev_gbx = {
 	gengbx_option,
 	gengbx_start,
-	(void(*)(float,float))gendev_null,
+	gendev_nogrid,
 	gengbx_arc,
 	gengbx_ellipse,
 	gengbx_line,
