@@ -1,8 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Parts Copyright (c) 1999 by T. Sato
- * Parts Copyright (c) 1999-2012 by Brian V. Smith
- * Parts Copyright (c) 2015-2017 by Thomas Loimer
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -41,10 +42,11 @@
 #endif
 #include <math.h>
 #include <ctype.h>
-#include "pi.h"
 
-#include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
+#include "messages.h"
+#include "pi.h"
 
 #define TEXT_LENGTH  300
 
@@ -157,14 +159,15 @@ add_link(char *area)
 void
 genmap_start(F_compound *objects)
 {
+  char *basename_buf = NULL;
   char *basename;
   char *ref;
   int i;
 
   if (to != NULL) {
-    basename = (char *)malloc(strlen(to) + 1);
-    strcpy(basename, to);
-    for (i = strlen(basename) - 1; 0 < i; i--) {
+    basename_buf = strdup(to);
+    basename = basename_buf;
+    for (i = strlen(basename) - 1; 0 < i; --i) {
       if (basename[i] == '.') {
 	basename[i] = '\0';
 	break;
@@ -192,6 +195,8 @@ genmap_start(F_compound *objects)
 	    XZOOM(llx), YZOOM(lly), XZOOM(urx), YZOOM(ury), ref);
     add_link(buf);
   }
+  if (basename_buf)
+    free(basename_buf);
 }
 
 int
@@ -388,13 +393,12 @@ genmap_text(F_text *t)
 struct driver dev_map = {
 	genmap_option,
 	genmap_start,
-	(void(*)(float,float))gendev_null,
+	gendev_nogrid,
 	genmap_arc,
 	genmap_ellipse,
 	genmap_line,
-	gendev_null,
+	(void (*)(F_spline *))gendev_null,
 	genmap_text,
 	genmap_end,
 	INCLUDE_TEXT
 };
-
