@@ -1,7 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -32,10 +34,29 @@
 #include <signal.h>
 
 #include "fig2dev.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "object.h"
 #include "genps.h"
+#include "messages.h"
+
+/*
+ * The ghostscript command line.
+ * With -dAutoFilterColorImages=false,
+ * -dColorImageFilter=/FlateEncode	produces a lossless but large image,
+ * -dColorImageFilter=/DCTEncode	produces a lossy but much smaller image.
+ * The default, -dAutoFilterColorImages=true is fine for e.g., png and jpg.
+ * -o also sets -dBATCH -dNOPAUSE
+ */
+#ifdef GSEXE
+#define	GSFMT	GSEXE " -q -dSAFER -sAutoRotatePages=None -sDEVICE=pdfwrite " \
+			"-dPDFSETTINGS=/prepress -o '%s' -"
+#else
+#define GSFMT	""
+#endif
+
+/* String buffer for the ghostscript command. 82 chars for the filename */
+static char	com_buf[sizeof GSFMT + 80];
+static char	*com = com_buf;
 
-static char	*com;
 
 void
 genpdf_option(char opt, char *optarg)
@@ -66,42 +87,42 @@ pdf_broken_pipe(int sig)
 void
 genpdf_start(F_compound *objects)
 {
-	size_t	n;
+#ifdef GSEXE
+	size_t	len;
 	char	*ofile;
 
 	/* divert output from ps driver to the pipe into ghostscript */
 	/* but first close the output file that main() opened */
 	if (tfp != stdout) {	/* equivalent to to != NULL */
 		fclose(tfp);
-		n = strlen(to);
 		ofile = to;
 	} else {
-		n = 0;
 		ofile = "-";
 	}
 
-	if ((com = malloc(n + 200)) == NULL) {
-	       fputs("Cannot allocate memory for ghostscript command string.\n",
-			stderr);
+	/* write command for conversion to pdf */
+	len = sizeof GSFMT + strlen(ofile) - 2;
+	if (len > sizeof com_buf && (com = malloc(len)) == NULL) {
+		put_msg(Err_mem);
 		exit(EXIT_FAILURE);
 	}
-	/* Notes about -ColorImageFilter:
-	 * /FlateEncode produces a lossless but large image
-	 * /DCTEncode produces a lossy but much smaller image
-	 */
-	/* -o ... is equivalent to -dBATCH -dNOPAUSE, see ghostscript Use.htm
-	   and -dBATCH is equivalent to -c quit */
-	sprintf(com, "%s -q -sAutoRotatePages=None "
-		"-dAutoFilterColorImages=false -dColorImageFilter=/DCTEncode "
-		"-sDEVICE=pdfwrite -dPDFSETTINGS=/prepress -o '%s' -",
-		GSEXE, ofile);
+	if (sprintf(com, GSFMT, ofile) < 0) {
+		err_msg("fig2dev: error when creating ghostscript command");
+		exit(EXIT_FAILURE);
+	}
+
 	(void) signal(SIGPIPE, pdf_broken_pipe);
 	if ((tfp = popen(com, "w")) == 0) {
-		fprintf(stderr, "fig2dev: Cannot open pipe to ghostscript\n");
-		fprintf(stderr, "command was: %s\n", com);
+		err_msg("fig2dev: Cannot open pipe to ghostscript");
+		put_msg("Command was: %s", com);
 		exit(EXIT_FAILURE);
 	}
 	genps_start(objects);
+#else /* GSEXE */
+	put_msg("This fig2dev is compiled without ghostscript support.\n"
+			"Cannot create pdf output.");
+	exit(EXIT_FAILURE);
+#endif
 }
 
 int
@@ -112,7 +133,8 @@ genpdf_end(void)
 	/* wrap up the postscript output */
 	if (genps_end() != 0) {
 		pclose(tfp);
-		free(com);
+		if (com != com_buf)
+			free(com);
 		return -1;		/* error, return now */
 	}
 
@@ -120,15 +142,15 @@ genpdf_end(void)
 	/* we've already closed the original output file */
 	tfp = 0;	/* so main() does not close tfp again */
 	if (status != 0) {
-		fprintf(stderr, "Error in ghostcript command\n");
-		fprintf(stderr, "command was: %s\n", com);
-		free(com);
-		return -1;
+		err_msg("Error in ghostcript command,\n  %s", com);
+		status = -1;
+	} else {
+		(void)signal(SIGPIPE, SIG_DFL);
 	}
-	(void) signal(SIGPIPE, SIG_DFL);
-	free(com);
 
-	return 0;
+	if (com != com_buf)
+		free(com);
+	return status;
 }
 
 struct driver dev_pdf = {
