@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -32,16 +32,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include <limits.h>
-#include "pi.h"
 
-#include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
-#include "texfonts.h"		/* texfontnames[] */
-#include "psfonts.h"
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
 #include "bound.h"
-#include "trans_spline.h"	/* create_line_with_spline() */
 #include "free.h"		/* free_linestorage() */
+#include "messages.h"
+#include "pi.h"
+#include "psfonts.h"
+#include "texfonts.h"		/* texfontnames[] */
+#include "trans_spline.h"	/* create_line_with_spline() */
 
 #undef M_PI_2
 #undef M_PI_4
@@ -116,7 +116,7 @@ static int	cur_thickness = 0;
 static int	saved_thickness = 0;
 static int	cur_joinstyle = 0;
 static int	cur_capstyle = 0;
-static int	default_color = BLACK_COLOR;
+static int	default_color = DEFAULT;
 static int	cur_color = DEFAULT;
 static int	cur_shade = NUMSHADES - 1;
 static int	border_margin = 0;
@@ -136,7 +136,7 @@ struct pict2earrow {
 	/* type 0 */
 	{ 0., 0.5},
 	/* place holder for what would be type 0 filled */
-	{ 0 },
+	{ 0., 0. },
 	/* type 1a simple triangle */
 	{ 1., 0.5},
 	/* type 1b filled simple triangle*/
@@ -198,7 +198,7 @@ struct pict2earrow {
 #define YCOORD(y)	(ury - (y))
 #define XDIR(x)		x
 #define YDIR(y)		-(y)
-#define EQUAL(p,q)	p->x == q->x && p->y == q->y
+#define EQUAL(p,q)	(p->x == q->x && p->y == q->y)
 			/* cast to double, to not have the integers overflow */
 #define LENGTH(p,q)	sqrt(((double) (q->x - p->x))*(q->x - p->x) \
 			     + ((double)(q->y - p->y))*(q->y - p->y))
@@ -529,23 +529,24 @@ set_fillcolor(int col, int shade, int *pen_color)
 	}
 
 	/* an unknown color can not be shaded or tinted */
-	if (col == DEFAULT)
+	if (col == DEFAULT) {
 	    if (default_color == DEFAULT)
 		col = BLACK_COLOR;
 	    else
 		col = default_color;
+	}
 
 	/* black */
-	if (shade == 0 && col != BLACK_COLOR
-		|| col == WHITE_COLOR && shade == NUMSHADES + NUMTINTS -1) {
+	if ((shade == 0 && col != BLACK_COLOR)
+		|| (col == WHITE_COLOR && shade == NUMSHADES + NUMTINTS -1)) {
 	    set_color(BLACK_COLOR);
 	    cur_shade = NUMSHADES - 1;
 	    return;
 	}
 
 	/* white */
-	if (shade == NUMSHADES + NUMTINTS - 1 && col != WHITE_COLOR
-		 || col == BLACK_COLOR && shade == 0) {
+	if ((shade == NUMSHADES + NUMTINTS - 1 && col != WHITE_COLOR)
+		 || (col == BLACK_COLOR && shade == 0)) {
 	    set_color(WHITE_COLOR);
 	    cur_shade = NUMSHADES - 1;
 	    return;
@@ -586,7 +587,7 @@ set_fillcolor(int col, int shade, int *pen_color)
 	get_rgbcolor(&col,&rgb);
 							/* 4080 = 255 << 4 */
 #define	SHADE(c,t)	(double)(c<<4)*t / (NUMSHADES-1) / 4080.
-#define	TINT(c,t)	((c<<4) + ((double)(255-c<<4) * (t-NUMSHADES+1) \
+#define	TINT(c,t)	((c<<4) + ((double)((255-c)<<4) * (t-NUMSHADES+1) \
 							/ NUMTINTS)) / 4080.
 	if (shade < NUMSHADES) {
 	    red = SHADE(rgb.red, shade);
@@ -2221,11 +2222,12 @@ put_font(F_text *t)
 	}
 
 	if (psfont_text(t))
-	    fprintf(tfp, "\\usefont%s",
-		    texpsfonts[t->font <= MAX_PSFONT ? t->font + 1 : 0]);
+		fprintf(tfp, "\\usefont%s", texpsfonts[t->font <= MAX_PSFONT ?
+				t->font + 1 : 0]);
 	else
-	    fprintf(tfp, "\\normalfont%s ",
-		    texfonts[t->font <= MAX_FONT ? t->font : MAX_FONT - 1]);
+		/* Default psfont is -1, default texfont 0, also accept -1. */
+		fprintf(tfp, "\\normalfont%s ", texfonts[t->font <= MAX_FONT ?
+				(t->font >= 0 ? t->font : 0) : MAX_FONT - 1]);
 }
 
 void
@@ -2287,7 +2289,7 @@ genpict2e_text(F_text *t)
 	    for (cp = (unsigned char*)t->cstring; *cp; ++cp) {
 		if (strchr("$&%#_{}", *cp))
 		    fputc('\\', tfp);
-		if (c = strchr("~^\\", *cp)) {
+		if ((c = strchr("~^\\", *cp))) {
 		    if (*c == '\\')
 			fputs("\\textbackslash ", tfp);
 		    else
@@ -2609,7 +2611,7 @@ genpict2e_arc(F_arc *a)
 struct driver dev_pict2e = {
 	genpict2e_option,
 	genpict2e_start,
-	(void(*)(float,float))gendev_null,
+	gendev_nogrid,
 	genpict2e_arc,
 	genpict2e_ellipse,
 	genpict2e_line,
