@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2017 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -33,12 +33,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
-#include "pi.h"
 
-#include "fig2dev.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
-#include "texfonts.h"
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
+#include "messages.h"
 #include "localmath.h"	/* arc_tangent() */
+#include "pi.h"
+#include "texfonts.h"
 
 static void putline();
 
@@ -87,7 +88,7 @@ gentextyl_option(char opt, char *optarg)
 		case 'l':			/* set line thickness */
 		    linethick = atoi(optarg);
 		    if (linethick < 1 || linethick > 12) {
-		      put_msg(Err_badarg, opt, "textyl");
+		      put_msg("Line thickness must be between 1 and 12.");
 		      exit(1);
 		    }
 		    break;
