@@ -1,10 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Copyright (c) 1998 by Mike Markowski
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2002 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -21,7 +20,7 @@
  * gentk: convert fig to Tk
  *
  * Author: Mike Markowski <mm@udel.edu>, U of Delaware, 4/98
- *         Modifications by Thomas Loimer, 2018
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,12 +34,13 @@
 #include <strings.h>
 #endif
 #include <math.h>
-#include <limits.h>
-#include "pi.h"
 
-#include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
 #include "colors.h"	/* lookup_X_color() */
+#include "messages.h"
+#include "pi.h"
+#include "readpics.h"
 #include "tkpattern.h"
 
 #define X(x) ((double)(x)/ppi)
@@ -160,7 +160,7 @@ void
 gentk_start(F_compound *objects)
 {
 	float		wid = -1., ht = -1., swap;
-	struct paperdef	*pd;
+	const struct paperdef	*pd;
 
 	fprintf(tfp, "# Produced by fig2dev Version %s\n", PACKAGE_VERSION);
 	ppi = ppi / mag * 80/72.0;
@@ -385,37 +385,35 @@ gentk_line(F_line *l)
 static void
 drawBitmap(F_line *l)
 {
-	char	stfp[256];
-	double	dx, dy;
-	F_pic	*p;
-	unsigned char	buf[16];
-	FILE	*fd;
-	int	filtype;	/* file (0) or pipe (1) */
-	int	stat;
-	FILE	*open_picfile(char *name, int *type,bool pipeok,char **retname);
-	void	close_picfile(FILE *file, int type);
-	char	*xname;
+	char			stfp[256];
+	double			dx, dy;
+	F_pic			*p;
+	unsigned char		buf[16];
+	int			stat;
+	struct xfig_stream	pic_stream;
 
 	p = l->pic;
 
 	dx = l->points->next->next->x - l->points->x;
 	dy = l->points->next->next->y - l->points->y;
 	if (!(dx >= 0. && dy >= 0.))
-		fprintf(stderr, "Rotated images not supported by Tk.\n");
+		put_msg("Rotated images not supported by Tk.");
 
 	/* see if GIF first */
 
-	if ((fd=open_picfile(p->file, &filtype, true, &xname)) == NULL) {
-		fprintf(stderr, "Can't open image file %s\n", p->file);
+	init_stream(&pic_stream);
+	if (open_stream(p->file, &pic_stream) == NULL) {
+		put_msg("Cannot open image file %s", p->file);
+		free_stream(&pic_stream);
 		return;
 	}
-	free(xname);	/* not needed */
 
 	/* read header */
-	stat = ReadOK(fd,buf,7);
+	stat = ReadOK(pic_stream.fp, buf, 7);
 	if (!stat) {
-		fprintf(stderr, "Image file %s too short\n", p->file);
-		close_picfile(fd, filtype);
+		put_msg("Image file %s too short", p->file);
+		close_stream(&pic_stream);
+		free_stream(&pic_stream);
 		return;
 	}
 
@@ -427,7 +425,6 @@ drawBitmap(F_line *l)
 		/* first make a name without the suffix */
 		char	*pname, *dot;
 
-		close_picfile(fd,filtype);
 		pname = strdup(p->file);
 		if ((dot=strchr(pname,'.')))
 			*dot='\0';
@@ -442,8 +439,9 @@ drawBitmap(F_line *l)
 	} else {
 		/* Try for an X Bitmap file format. */
 		unsigned int dummy;
-		rewind(fd);
-		if (ReadFromBitmapFile(fd, &dummy, &dummy, &p->bitmap)) {
+		rewind_stream(&pic_stream);
+		if (ReadFromBitmapFile(pic_stream.fp, &dummy, &dummy,
+								&p->bitmap)) {
 			sprintf(stfp, "%s create bitmap %fi %fi -anchor nw",
 					canvas,
 					X(l->points->x), Y(l->points->y));
@@ -464,10 +462,11 @@ drawBitmap(F_line *l)
 			}
 			fputc('\n', tfp);
 		} else
-			fprintf(stderr, "Only X bitmap and GIF picture objects "
-					"are supported in Tk canvases.\n");
-		close_picfile(fd,filtype);
+			put_msg("Only X bitmap and GIF picture objects "
+					"are supported in Tk canvases.");
 	}
+	close_stream(&pic_stream);
+	free_stream(&pic_stream);
 }
 
 /*
@@ -1425,7 +1424,7 @@ niceLine(char *s)
 struct driver	dev_tk = {
 	gentk_option,
 	gentk_start,
-	(void(*)(float,float))gendev_null,
+	gendev_nogrid,
 	gentk_arc,
 	gentk_ellipse,
 	gentk_line,
