@@ -1,7 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -42,26 +44,22 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include <limits.h>
 
 #include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "object.h"
+#include "messages.h"
+#include "readpics.h"
+#include "xtmpfile.h"
 
-extern	FILE	*open_picfile(char *, int *, bool, char **);	/* readpics.c */
 extern	int	 _read_pcx(FILE *pcxfile, F_pic *pic);		/* readpcx.c */
 
-static bool	 ReadColorMap(FILE *, unsigned int,
-				unsigned char cmap[3][MAXCOLORMAPSIZE]);
-static bool	 DoGIFextension(FILE *, int);
-static int	 GetDataBlock(FILE *, unsigned char *);
-
 #define LOCALCOLORMAP		0x80
 #define	ReadOK(file,buffer,len)	(fread(buffer, len, 1, file) != 0)
 #define BitSet(byte, bit)	(((byte) & (bit)) == (bit))
 
 #define LM_to_uint(a,b)			(((b)<<8)|(a))
 
-struct {
+struct _GifScreen {
 	unsigned int	Width;
 	unsigned int	Height;
 	unsigned int	BitPixel;
@@ -69,47 +67,76 @@ struct {
 	unsigned int	ColorResolution;
 	unsigned int	Background;
 	unsigned int	AspectRatio;
-} GifScreen;
+};
 
-struct {
+struct _Gif89 {
 	int	transparent;
 	int	delayTime;
 	int	inputFlag;
 	int	disposal;
-} Gif89 = { -1, -1, -1, 0 };
+} Gif89;
+
+static bool	 ReadColorMap(FILE *, unsigned int,
+				unsigned char cmap[3][MAXCOLORMAPSIZE]);
+static bool	 DoGIFextension(FILE *, struct _Gif89 *, int);
+static int	 GetDataBlock(FILE *, unsigned char *);
 
 /* return codes:  1 : success
 		  0 : invalid file
 */
 
 int
-read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
+read_gif(F_pic *pic, struct xfig_stream *restrict pic_stream, int *llx,int *lly)
 {
-	char		 buf[512];
-	char		 *realname, *cmd;
-	FILE		*file, *giftopcx;
+	char		buf[BUFSIZ];
+	char		pcxname_buf[128] = "f2dpcxXXXXXX";
+	char		*pcxname = pcxname_buf;
+	char		*cmd = buf;
+	static char	*cmd_fmt = NULL;
 	int		 i, stat;
 	int		 useGlobalColormap;
 	unsigned int	 bitPixel;
 	unsigned char	 c;
+	size_t		size;
 	char		 version[4];
 	unsigned char    transp[3]; /* RGB of transparent color (if any) */
+	FILE		*pcx;
+	FILE		*giftopcx;
+	struct _Gif89	Gif89 = { -1, -1, -1, 0};
+	struct _GifScreen	GifScreen;
 
-	/* open the file */
-	if ((file=open_picfile(filename, &filetype, false,&realname)) == NULL) {
-		fprintf(stderr,"No such GIF file: %s\n", filename);
+	if (!rewind_stream(pic_stream))
 		return 0;
-	}
 
-	/* first read header to look for any transparent color extension */
+	/* command string to convert gif to pcx */
+
+	if (cmd_fmt == NULL) {
+		if (!system("{ giftopnm -version && ppmtopcx -version; } "
+								"2>/dev/null"))
+			cmd_fmt = "giftopnm -quiet | ppmtopcx -quiet >'%s'";
+		else if (!system("convert -version >/dev/null"))
+			cmd_fmt = "convert - pcx:'%s'";
+		else if (!system("gm -version >/dev/null"))
+			cmd_fmt = "gm convert - pcx:'%s'";
+		else {
+			cmd_fmt = "";
+			put_msg("Cannot read gif files.\n"
+	"To read gif files, install either the netpbm, or the imagemagick,\n"
+	"or the graphicsmagick package.");
+			return 0;
+		}
+	}
+	if (*cmd_fmt == '\0')
+		return 0;
 
 	*llx = *lly = 0;
 
-	if (! ReadOK(file,buf,6)) {
+	/* first read header to look for any transparent color extension */
+	if (! ReadOK(pic_stream->fp, buf, 6)) {
 		return 0;
 	}
 
-	if (strncmp((char*)buf,"GIF",3) != 0) {
+	if (strncmp((char*)buf, "GIF", 3) != 0) {
 		return 0;
 	}
 
@@ -117,11 +144,11 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 	version[3] = '\0';
 
 	if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0)) {
-		fprintf(stderr,"Unknown GIF version %s\n",version);
+		put_msg("Unknown GIF version %s", version);
 		return 0;
 	}
 
-	if (! ReadOK(file,buf,7)) {
+	if (! ReadOK(pic_stream->fp, buf, 7)) {
 		return 0;		/* failed to read screen descriptor */
 	}
 
@@ -133,7 +160,8 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 	GifScreen.AspectRatio     = (unsigned int) buf[6];
 
 	if (BitSet(buf[4], LOCALCOLORMAP)) {	/* Global Colormap */
-		if (!ReadColorMap(file,GifScreen.BitPixel,pic->cmap)) {
+		if (!ReadColorMap(pic_stream->fp, GifScreen.BitPixel,
+								pic->cmap)) {
 			return 0;	/* error reading global colormap */
 		}
 	}
@@ -142,7 +170,7 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 	Gif89.transparent =  -1;
 
 	for (;;) {
-		if (! ReadOK(file,&c,1)) {
+		if (! ReadOK(pic_stream->fp, &c, 1)) {
 			return 0;	/* EOF / read error on image data */
 		}
 
@@ -151,10 +179,10 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 		}
 
 		if (c == '!') {		/* Extension */
-		    if (! ReadOK(file,&c,1))
+		    if (! ReadOK(pic_stream->fp, &c, 1))
 			fprintf(stderr,
 				"GIF read error on extension function code\n");
-		    (void) DoGIFextension(file, c);
+		    (void)DoGIFextension(pic_stream->fp, &Gif89, c);
 		    continue;
 		}
 
@@ -162,8 +190,8 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 			continue;
 		}
 
-		if (! ReadOK(file,buf,9)) {
-			return 1;      /* couldn't read left/top/width/height */
+		if (! ReadOK(pic_stream->fp, buf, 9)) {
+			return 0;      /* couldn't read left/top/width/height */
 		}
 
 		useGlobalColormap = ! BitSet(buf[8], LOCALCOLORMAP);
@@ -171,9 +199,9 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 		bitPixel = 1<<((buf[8]&0x07)+1);
 
 		if (! useGlobalColormap) {
-		    if (!ReadColorMap(file, bitPixel, pic->cmap)) {
+		    if (!ReadColorMap(pic_stream->fp, bitPixel, pic->cmap)) {
 			fprintf(stderr, "error reading local GIF colormap\n");
-			return 1;
+			return 0;
 		    }
 		}
 		break;			/* image starts here, header is done */
@@ -183,48 +211,84 @@ read_gif(char *filename, int filetype, F_pic *pic, int *llx, int *lly)
 	fprintf(tfp, "%% Originally from a GIF File: %s\n\n", pic->file);
 
 	/* save transparent indicator */
-	pic->transp = Gif89.transparent;
-	/* and RGB values */
-	if (pic->transp != -1) {
-	    transp[RED]   = pic->cmap[RED][pic->transp];
-	    transp[GREEN] = pic->cmap[GREEN][pic->transp];
-	    transp[BLUE]  = pic->cmap[BLUE][pic->transp];
+	if (Gif89.transparent != -1) {
+		pic->num_transp = 1;
+		pic->transp_cols = pic->transp_col; /* abuse transp_col array */
+		pic->transp_cols[0] = Gif89.transparent;
+		transp[RED]   = pic->cmap[RED][pic->transp_cols[0]];
+		transp[GREEN] = pic->cmap[GREEN][pic->transp_cols[0]];
+		transp[BLUE]  = pic->cmap[BLUE][pic->transp_cols[0]];
+	} else {
+		pic->num_transp = NO_TRANSPARENCY;
 	}
 
-	/* reposition the file at the beginning */
-	fseek(file, 0, SEEK_SET);
+	/* create a temporary file */
+	if ((pcx = xtmpfile(&pcxname, sizeof pcxname_buf)) == NULL) {
+		if (pcxname != pcxname_buf)
+			free(pcxname);
+		return 0;
+	}
+#define REMOVE_PCXFILE			\
+	do {				\
+		fclose(pcx);		\
+		unlink(pcxname);	\
+		if (pcxname != pcxname_buf) \
+			free(pcxname);	\
+	} while (0)
+
 
 	/* now call giftopnm and ppmtopcx */
-	if ((cmd = malloc(strlen(realname) + 50)) == NULL) {
-		fputs("Not enough memory to store command string.\n", stderr);
-		free(realname);
+	if ((size = strlen(cmd_fmt) + strlen(pcxname) > sizeof buf) &&
+			(cmd = malloc(size - 1)) == NULL) {
+		REMOVE_PCXFILE;
 		return 0;
 	}
-	sprintf(cmd, "giftopnm -quiet %s | ppmtopcx -quiet 2>/dev/null",
-		realname);
-	free(realname);
-	if ((giftopcx = popen(cmd, "r")) == 0) {
-		fprintf(stderr, "Cannot open pipe from giftopnm | ppmtopcx.\n");
+	sprintf(cmd, cmd_fmt, pcxname);
+
+	giftopcx = popen(cmd, "w");
+	if (giftopcx == NULL) {
+		err_msg("Cannot convert gif to pcx, %s", cmd);
+		REMOVE_PCXFILE;
+		if (cmd != buf)
+			free(cmd);
+		return 0;
+	}
+	if (cmd != buf)
 		free(cmd);
+
+	/* rewind gif stream */
+	if (!rewind_stream(pic_stream)) {
+		REMOVE_PCXFILE;
 		return 0;
 	}
-	free(cmd);
+	while ((size = fread(buf, 1, sizeof buf, pic_stream->fp)) != 0)
+		fwrite(buf, size, 1, giftopcx);
+
+	if (pclose(giftopcx)) {
+		err_msg("Could not read gif file '%s'", pic_stream->name);
+		REMOVE_PCXFILE;
+		return 0;
+	}
+
+	rewind(pcx);
 	/* now call _read_pcx to read the pcx file */
-	stat = _read_pcx(giftopcx, pic);
-	/* close file */
-	pclose(giftopcx);
+	stat = _read_pcx(pcx, pic);
+
+	REMOVE_PCXFILE;
 
 	/* now match original transparent colortable index with possibly new
 	   colortable from ppmtopcx */
-	if (pic->transp != -1) {
+	if (pic->num_transp != NO_TRANSPARENCY) {
 	    for (i=0; i<pic->numcols; i++) {
 		if (pic->cmap[RED][i]   == transp[RED] &&
 		    pic->cmap[GREEN][i] == transp[GREEN] &&
-		    pic->cmap[BLUE][i]  == transp[BLUE])
+		    pic->cmap[BLUE][i]  == transp[BLUE]) {
+			pic->transp_cols[0] = i;
 			break;
+		}
 	    }
-	    if (i < pic->numcols)
-		pic->transp = i;
+	    if (i == pic->numcols)	/* not found? */
+		pic->num_transp = NO_TRANSPARENCY;
 	}
 
 	return stat;
@@ -238,49 +302,49 @@ ReadColorMap(FILE *fd, unsigned int number,
 	unsigned char	rgb[3];
 
 	for (i = 0; i < number; ++i) {
-	    if (! ReadOK(fd, rgb, sizeof(rgb))) {
-		fprintf(stderr,"bad GIF colormap\n" );
-		return false;
-	    }
-	    cmap[RED][i]   = rgb[RED];
-	    cmap[GREEN][i] = rgb[GREEN];
-	    cmap[BLUE][i]  = rgb[BLUE];
+		if (! ReadOK(fd, rgb, sizeof(rgb))) {
+			fprintf(stderr,"bad GIF colormap\n" );
+			return false;
+		}
+		cmap[RED][i]   = rgb[RED];
+		cmap[GREEN][i] = rgb[GREEN];
+		cmap[BLUE][i]  = rgb[BLUE];
 	}
 	return true;
 }
 
 static bool
-DoGIFextension(FILE *fd, int label)
+DoGIFextension(FILE *fd, struct _Gif89 *Gif89, int label)
 {
 	static unsigned char buf[256];
 	char	    *str;
 
 	switch (label) {
-	    case 0x01:		/* Plain Text Extension */
+	case 0x01:		/* Plain Text Extension */
 		str = "Plain Text Extension";
 		break;
-	    case 0xff:		/* Application Extension */
+	case 0xff:		/* Application Extension */
 		str = "Application Extension";
 		break;
-	    case 0xfe:		/* Comment Extension */
+	case 0xfe:		/* Comment Extension */
 		str = "Comment Extension";
 		while (GetDataBlock(fd, buf) != 0) {
 			; /* GIF comment */
 		}
 		return false;
-	    case 0xf9:		/* Graphic Control Extension */
+	case 0xf9:		/* Graphic Control Extension */
 		str = "Graphic Control Extension";
 		(void) GetDataBlock(fd, (unsigned char*) buf);
-		Gif89.disposal    = (buf[0] >> 2) & 0x7;
-		Gif89.inputFlag   = (buf[0] >> 1) & 0x1;
-		Gif89.delayTime   = LM_to_uint(buf[1],buf[2]);
+		Gif89->disposal    = (buf[0] >> 2) & 0x7;
+		Gif89->inputFlag   = (buf[0] >> 1) & 0x1;
+		Gif89->delayTime   = LM_to_uint(buf[1],buf[2]);
 		if ((buf[0] & 0x1) != 0)
-			Gif89.transparent = buf[3];
+			Gif89->transparent = buf[3];
 
 		while (GetDataBlock(fd, buf) != 0)
 			;
 		return false;
-	    default:
+	default:
 		str = (char *) buf;
 		sprintf(str, "UNKNOWN (0x%02x)", label);
 		break;
@@ -292,8 +356,6 @@ DoGIFextension(FILE *fd, int label)
 	return false;
 }
 
-int	ZeroDataBlock = false;
-
 static int
 GetDataBlock(FILE *fd, unsigned char *buf)
 {
@@ -304,8 +366,6 @@ GetDataBlock(FILE *fd, unsigned char *buf)
 		return -1;
 	}
 
-	ZeroDataBlock = count == 0;
-
 	/* error in reading DataBlock */
 	if ((count != 0) && (! ReadOK(fd, buf, count))) {
 		return -1;
