@@ -1,8 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Parts Copyright (c) 1992 by Brian Boyter
- * Parts Copyright (c) 1992-2007 by Brian V. Smith
- * Parts Copyright (c) 2015-2017 by Thomas Loimer
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -30,8 +31,9 @@
 #include <string.h>
 
 #include "fig2dev.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "object.h"
 #include "colors.h"	/* rgb2luminance() */
+#include "readpics.h"
 
 extern int _read_pcx(FILE *pcxfile, F_pic *pic);
 
@@ -40,13 +42,12 @@ extern int _read_pcx(FILE *pcxfile, F_pic *pic);
 */
 
 int
-read_pcx(FILE *file, int filetype, F_pic *pic, int *llx, int *lly)
+read_pcx(F_pic *pic, struct xfig_stream *restrict pic_stream, int *llx,int *lly)
 {
-	(void)	filetype;
-
+	if (!rewind_stream(pic_stream))
+		return 0;
 	*llx = *lly = 0;
-	pic->transp = -1;
-	return _read_pcx(file, pic);
+	return _read_pcx(pic_stream->fp, pic);
 }
 
 /* pcx2ppm 1.2 - convert pcx to ppm
@@ -143,7 +144,8 @@ _read_pcx(FILE *pcxfile, F_pic *pic)
 	if (real_bpp==1 || real_bpp==4)
 	    bytemax=(1<<30); */	/* we use a 'y<h' test instead for these files */
 
-	if ((pic->bitmap=malloc(w*(h+2)*bytepp))==NULL)
+	/* dispbyte() seems to reach beyond a line - malloc() a bit more */
+	if ((pic->bitmap=malloc((w < 8 ? 8 : w)*(h+2)*bytepp))==NULL)
 	    return 0;
 
 	/* need this if more than one bitplane */
@@ -178,11 +180,6 @@ _read_pcx(FILE *pcxfile, F_pic *pic)
 	/* read palette */
 	switch(real_bpp) {
 	    case 1:
-		pic->cmap[RED][0] = pic->cmap[GREEN][0] = pic->cmap[BLUE][0] = 0;
-		pic->cmap[RED][1] = pic->cmap[GREEN][1] = pic->cmap[BLUE][1] = 255;
-		pic->numcols = 2;
-		break;
-
 	    case 2:
 	    case 3:
 	    case 4:
@@ -295,7 +292,7 @@ dispbyte(unsigned char *ptr,int *xp,int *yp,int c,int w,int h,
 		break;
 
 	  case 24:
-		*(ptr+((*yp)*w+*xp)*3+(2-(*planep)))=c;
+		*(ptr+((*yp)*w+*xp)*3+*planep)=c;
 		(*xp)++;
 		if (*xp>=w) {
 		    (*xp)=0;
