@@ -1,7 +1,9 @@
 /*
  * Fig2dev: Translate Fig code to various Devices
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
- * Parts Copyright (c) 2015-2017 by Thomas Loimer
+ * Copyright (c) 1991 by Micah Beck
+ * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -63,12 +65,13 @@
 #include <ctype.h>
 
 #include "fig2dev.h"	/* includes "bool.h" */
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "object.h"
+#include "readpics.h"
 
 #define MAX_SIZE 255
 
-int ReadFromBitmapFile (FILE *file, unsigned int *width, unsigned int *height,
-		unsigned char **data_ret);
+int	ReadFromBitmapFile (FILE *file, unsigned int *width,
+				unsigned int *height, unsigned char **data_ret);
 /* attempt to read a bitmap file */
 
 /* return codes:  1 : success
@@ -76,15 +79,17 @@ int ReadFromBitmapFile (FILE *file, unsigned int *width, unsigned int *height,
 */
 
 int
-read_xbm(FILE *file, int filetype, F_pic *pic, int *llx, int *lly)
+read_xbm(F_pic *pic, struct xfig_stream *restrict pic_stream, int *llx,int *lly)
 {
-    (void) filetype;
     int status;
     unsigned int x, y;
 
+    if (!rewind_stream(pic_stream))
+	return 0;
     *llx = *lly = 0;
+
     /* first try for a X Bitmap file format */
-    status = ReadFromBitmapFile(file, &x, &y, &pic->bitmap);
+    status = ReadFromBitmapFile(pic_stream->fp, &x, &y, &pic->bitmap);
     if (status == 1) {
 	pic->subtype = P_XBM;
 	pic->hw_ratio = (float) y / x;
@@ -213,7 +218,7 @@ ReadFromBitmapFile (FILE *file,
 	initHexTable();
 
     /* error cleanup and return macro	*/
-#define	RETURN(code) { if (data) free (data);  return code; }
+#define	RETURN(code) do {if (data) free(data);  return code;} while (0)
 
     while (fgets(line, MAX_SIZE, file)) {
 	if (strlen(line) == MAX_SIZE-1) {
@@ -260,9 +265,8 @@ ReadFromBitmapFile (FILE *file,
 	bytes_per_line = (ww+7)/8 + padding;
 
 	size = bytes_per_line * hh;
-	data = (unsigned char *) malloc ((unsigned int) size);
-	if (!data)
-	  RETURN (0);
+	if ((data = malloc((unsigned int) size)) == NULL)
+	  return 0;
 
 	if (version10p) {
 	    unsigned char *ptr;
@@ -288,12 +292,12 @@ ReadFromBitmapFile (FILE *file,
     }					/* end while */
 
     if (data == NULL) {
-	RETURN (0);
+	return 0;
     }
 
     *data_ret = data;
     *width = ww;
     *height = hh;
 
-    RETURN (1);
+    return 1;
 }
