@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2021 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -29,6 +29,9 @@
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <locale.h>
 /* In Windows, _setmode() is declared in <io.h>, O_BINARY in <fcntl.h>. It
  * accepts two arguments and sets file mode to text or binary. */
@@ -37,20 +40,19 @@
 #include <fcntl.h>
 #endif
 
-#include "fig2dev.h"	/* includes "bool.h" */
+#include "fig2dev.h"	/* includes bool.h and object.h */
+//#include "object.h"
 #include "alloc.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
-#include "drivers.h"
 #include "bound.h"
+#include "drivers.h"
+#include "messages.h"
 #include "read.h"
 
-/* the three lines below could go into a getopt.h file */
-extern int	 fig_getopt(int nargc, char **nargv, char *ostr); /* getopt.c */
-extern char	*optarg;		/* getopt.c */
-extern int	 optind;		/* getopt.c */
-
-char	Err_badarg[] = "Argument -%c unknown to %s driver.";
-char	Err_mem[] = "Running out of memory.";
+#ifndef HAVE_GETOPT
+extern int	getopt(int argc, char *argv[], const char *ostr);
+extern char	*optarg;
+extern int	 optind;
+#endif
 
 const char	prog[] = PACKAGE_NAME;
 char	*from = NULL, *to = NULL;
@@ -80,20 +82,18 @@ bool	bgspec = false;		/* flag to say -g was specified */
 #ifdef I18N
 bool support_i18n = false;
 #endif
-char	gif_transparent[20]="\0"; /* GIF transp color hex name (e.g. #ff00dd) */
-char	papersize[20];		/* paper size */
+char	gif_transparent[]="\0"; /* GIF transp color hex name (e.g. #ff00dd) */
+char	papersize[PAPERSZ_LEN];	/* paper size */
 char	boundingbox[64];	/* boundingbox */
-char	lang[40];		/* selected output language */
+char	lang[12];		/* selected output language */
 RGB	background;		/* background (if specified by -g) */
 float	grid_minor_spacing = 0.0; /* grid minor spacing (if any) */
 float	grid_major_spacing = 0.0; /* grid major spacing (if any) */
 float	THICK_SCALE;		/* convert line thickness from screen resolution
 					calculated in read_objects() */
 char	gscom[1000];		/* to build up a command for ghostscript */
-bool	psencode_header_done = false; /* if PSencode header was already emitted*/
-bool	transp_header_done = false;   /* if transparent image header was already emitted */
 
-char	*Fig_color_names[] = {	/* hex names for Fig colors */
+const char	*Fig_color_names[] = {	/* hex names for Fig colors */
 	"#000000", "#0000ff", "#00ff00", "#00ffff",
 	"#ff0000", "#ff00ff", "#ffff00", "#ffffff",
 	"#000090", "#0000b0", "#0000d0", "#87ceff",
@@ -103,7 +103,7 @@ char	*Fig_color_names[] = {	/* hex names for Fig colors */
 	"#803000", "#a04000", "#c06000", "#ff8080",
 	"#ffa0a0", "#ffc0c0", "#ffe0e0", "#ffd700"
 };
-struct paperdef paperdef[] = {
+const struct paperdef paperdef[] = {
 	{"Letter", 612, 792},		/*  8.5" x 11" */
 	{"Legal", 612, 1008},		/*  8.5" x 14" */
 	{"Tabloid", 792, 1224},		/*   11" x 17" */
@@ -161,9 +161,9 @@ static struct depth_opts {
 
 static char	Usage[] =
 #ifdef I18N
-"Usage:\n %s -hV\n %s -L language [-s size] [-m scale] [-j] [input [output]]\n";
+"Usage:\n %1$s -hV\n %1$s -L language [-s size] [-m scale] [-j] [input [output]]\n";
 #else
-    "Usage:\n %s -hV\n %s -L language [-s size] [-m scale] [input [output]]\n";
+"Usage:\n %1$s -hV\n %1$s -L language [-s size] [-m scale] [input [output]]\n";
 #endif
 
 static int	 parse_gridspec(char *string, float *numer, float *denom,
@@ -174,16 +174,6 @@ static void	 help_msg(void);
 static void	 depth_option(char *s);
 
 
-void
-put_msg(char *fmt, ...)
-{
-	va_list argptr;
-	va_start(argptr, fmt);
-	vfprintf(stderr, fmt, argptr);
-	va_end(argptr);
-	fputc('\n', stderr);
-}
-
 /*
  * print comments to the output file preceded by string1 and
  * succeeded by string2
@@ -218,10 +208,34 @@ get_args(int argc, char *argv[])
 	char	*grid, *p;
 	float	 numer, denom;
 
-	if (argc == 1)
-	    fprintf(stderr, Usage, prog, prog);
+	if (argc == 1) {
+		fprintf(stderr, Usage, prog);
+		exit(EXIT_SUCCESS);
+	}
+
+	/* print the version, for the comfort of the autotest tests */
+	if (!strcmp(argv[1], "--version")) {
+		printf("fig2dev Version %s\n", PACKAGE_VERSION);
+		exit(EXIT_SUCCESS);
+	}
+
+	/* Guess an output driver from the last argument */
+	if ((p = strrchr(argv[argc - 1], '.'))) {
+		++p;
+		for (i = 0; *drivers[i].name; ++i) {
+			if (!strcmp(p, drivers[i].name) || (drivers[i].alias &&
+						!strcmp(p, drivers[i].alias))) {
+				strcpy(lang, drivers[i].name);
+				dev = drivers[i].dev;
+				dev->option('L', optarg);
+				break;
+			}
+		}
+	}
+
+
 	/* sum of all arguments */
-	while ((c = fig_getopt(argc, argv, ARGSTRING)) != EOF) {
+	while ((c = getopt(argc, argv, ARGSTRING)) != EOF) {
 
 	  /* global (all drivers) option handling */
 	    switch (c) {
@@ -297,7 +311,10 @@ get_args(int argc, char *argv[])
 
 	    case 'L':			/* set output language */
 		/* save language for gen{gif,jpg,pcx,xbm,xpm,ppm,tif} */
-		strncpy(lang,optarg,sizeof(lang)-1);
+		strncpy(lang, optarg, sizeof(lang)-1);
+		lang[sizeof lang - 1] = '\0';
+		/* override if a language already was set above */
+		dev = NULL;
 		for (i = 0; *drivers[i].name; ++i)
 		    if (!strcmp(lang, drivers[i].name))
 			dev = drivers[i].dev;
@@ -420,22 +437,25 @@ main(int argc, char *argv[])
 
 	/* read the Fig file */
 
-	if (from)
+	if (from && strcmp(from, "-"))
 		status = read_fig(from, &objects);
-	else	/* read from stdin */
+	else
 		status = readfp_fig(stdin, &objects);
+	if (status != 0) {
+		if (status == -3) {
+			if (from && strcmp(from, "-"))
+				err_msg("File \"%s\" is not accessible", from);
+			else
+				err_msg("Input error");
+		}
+		exit(EXIT_FAILURE);
+	}
 
 	/* multiply grid spacing by unit and scale to get FIG units */
 	grid_minor_spacing = mult * grid_minor_spacing * ppi;
 	grid_major_spacing = mult * grid_major_spacing * ppi;
 
-	if (status != 0) {
-		if (from)
-			read_fail_message(from, status);
-		exit(1);
-	}
-
-	if (to == NULL)
+	if (to == NULL || !strcmp(to, "-"))
 		tfp = stdout;
 	else {
 		if (strcmp(to + strlen(to) - 4, ".fig") == 0 ){
@@ -491,9 +511,10 @@ help_msg(void)
 	int i, n;
 
 	fputs(	"General Options (all drivers):\n"
-		"  -L language	choose output language (this must be first)\n"
-		"                Available languages are:\n"
-		"                   ",
+"  -L language	choose output language (this must be first, if the language\n"
+"                cannot be inferrred from the suffix of the output file)\n"
+"                Available languages are:\n"
+"                   ",
 		stdout);
 	for (i = 0, n = 20; *drivers[i].name; ++i) {
 		if (n > 72) {
@@ -735,8 +756,8 @@ help_msg(void)
 			puts(
 "PICT2E Options:\n"
 "  -b width    specify width of blank border around figure (points, 1/72 inch)\n"
-"  -C num      do not issue color commands for color num (0 = black, 1 = red,\n"
-"                see xfig color chooser)\n"
+"  -C num      do not issue color commands for color num (0 = black,\n"
+"                1 = blue, etc., see xfig color chooser)\n"
 "  -e          do not try to be compatible with epic/eepic\n"
 "  -E num      set encoding for text translation (0 = no translation,\n"
 "                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
@@ -838,6 +859,7 @@ help_msg(void)
 "  -l dummyarg landscape mode (dummy argument required after \"-l\")\n"
 "  -p dummyarg portrait mode (dummy argument required after \"-p\")\n"
 "  -P          generate canvas of full page size instead of figure bounds\n"
+"  -w          add wrapper to generate complete perl file\n"
 "  -z papersize        set the papersize (see man pages for available sizes)"
 			);
 		}
@@ -846,11 +868,10 @@ help_msg(void)
 			puts(
 "TIKZ Options:\n"
 "  -b width    specify width of blank border around figure (1/72 inch)\n"
-"  -C num      do not issue color commands for color num\n"
-"                (0 = black, 1 = red, etc.)\n"
+"  -C num      do not issue color commands for color num (0 = black,\n"
+"                1 = blue, etc., see xfig color chooser)\n"
 "  -E num      set encoding for text translation (0 = no translation,\n"
 "                1 = ISO-8859-1, 2 = ISO-8859-2; default 1)\n"
-"  -e          do not try to be compatible with epic/eepic\n"
 "  -F          do not set font family/series/shape\n"
 "  -i dir      prepend the string \"dir\" to included graphics files\n"
 "  -O          do not quote characters special to TeX/LaTeX\n"
@@ -967,10 +988,16 @@ static int compound_dump(F_compound *com, struct obj_rec *array,
 	return count;
 }
 
+static int
+rec_comp(struct obj_rec *r1, struct obj_rec *r2)
+{
+	return (r2->depth - r1->depth);
+}
+
 int
 gendev_objects(F_compound *objects, struct driver *dev)
 {
-	int	obj_count, rec_comp();
+	int	obj_count;
 	int	status;
 	struct	obj_rec *rec_array, *r;
 
@@ -984,7 +1011,8 @@ gendev_objects(F_compound *objects, struct driver *dev)
 	(void)compound_dump(objects, rec_array, 0, dev);
 
 	/* sort object array by depth */
-	qsort(rec_array, obj_count, sizeof(struct obj_rec), rec_comp);
+	qsort(rec_array, obj_count, sizeof(struct obj_rec),
+			(int (*)(const void *, const void *))rec_comp);
 
 	/* generate header */
 	(*dev->start)(objects);
@@ -1005,16 +1033,18 @@ gendev_objects(F_compound *objects, struct driver *dev)
 	return status;
 }
 
-int rec_comp(struct obj_rec *r1, struct obj_rec *r2)
-{
-	return (r2->depth - r1->depth);
-}
-
 /* null operations */
 void
 gendev_null(void)
 { ; }
 
+void
+gendev_nogrid(float major, float minor)
+{
+	(void)major;
+	(void)minor;
+}
+
 /*
  * depth_options:
  *  +range[,range...]
