@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2019 by Thomas Loimer
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -33,8 +33,9 @@
 
 #define round(x)	((int) ((x) + ((x >= 0)? 0.5: -0.5)))
 
-#define	NUM_STD_COLS	32
-#define	MAX_USR_COLS	512
+#define NUM_STD_COLS	32
+#define MAX_USR_COLS	512
+#define PAPERSZ_LEN	16
 
 #define NUMARROWS	30	/* Synchronize with arrow_shapes[] in bound.c */
 
@@ -66,13 +67,9 @@ struct driver {
 
 extern void	gendev_null(void);
 extern void	gendev_nogrid(float major, float minor);
-extern void	put_msg(char *fmt, ...);
 extern void	print_comments(char *string1, F_comment *comment,char *string2);
 extern int	depth_filter(int obj_depth);
 
-extern char	Err_badarg[];
-extern char	Err_mem[];
-
 extern const char	prog[];
 extern char	*from, *to;
 extern char	*name;
@@ -100,8 +97,8 @@ extern bool	bgspec;		/* flag to say -g was specified */
 #ifdef I18N
 extern bool support_i18n;
 #endif
-extern char	gif_transparent[];/* GIF transp color hex name (e.g. #ff00dd) */
-extern char	papersize[];	/* paper size */
+extern char	gif_transparent[8];/* GIF transp color hex name (e.g. #ff00dd)*/
+extern char	papersize[PAPERSZ_LEN];	/* paper size */
 extern char	boundingbox[];	/* boundingbox */
 extern char	lang[];		/* selected output language */
 extern const char	*Fig_color_names[]; /* hex names for Fig colors */
