@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
- * Parts Copyright (c) 2015 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,14 +16,11 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include <stdio.h>
 #include <stdlib.h>
 
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "object.h"
 #include "free.h"
 
 void
@@ -151,9 +148,6 @@ free_linestorage(F_line *l)
 	if (l->pic) {
 		free(l->pic->file);
 		free(l->pic->bitmap);
-#ifdef HAVE_X11_XPM_H
-		XpmFreeXpmImage(&l->pic->xpmimage);
-#endif
 #ifdef V4_0
 		free_compound(l->pic->compound);
 #endif
