@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2007 by Brian V. Smith
- * Parts Copyright (c) 2015,2017 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,6 +16,9 @@
  *
  */
 
+#ifndef FREE_H
+#define FREE_H
+
 void free_arc(F_arc **list);
 void free_compound(F_compound **list);
 void free_ellipse(F_ellipse **list);
@@ -25,3 +28,5 @@ void free_spline(F_spline **list);
 void free_splinestorage(F_spline *s);
 void free_linestorage(F_line *l);
 void free_comments(F_comment *c);
+
+#endif /* FREE_H */
