@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2002 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,9 +16,10 @@
  *
  */
 
-#ifdef HAVE_X11_XPM_H
-#include <X11/xpm.h>
-#endif
+#ifndef OBJECT_H
+#define OBJECT_H
+
+#include <limits.h>	/* INT_MIN, INT_MAX */
 
 #define		BLACK_COLOR	0
 #define		WHITE_COLOR	7
@@ -40,6 +41,8 @@ typedef struct f_point {
 typedef struct f_pos {
 	int			x, y;
 } F_pos;
+#define COORD_MIN	INT_MIN
+#define COORD_MAX	INT_MAX
 
 typedef struct f_arrow {
 	int			type;
@@ -56,9 +59,9 @@ typedef struct f_comment {
 
 #define COMMON_PROPERTIES(o)						\
 	o->style < SOLID_LINE || o->style > DASH_3_DOTS_LINE ||		\
-	o->thickness < 0 || o->depth < 0 || o->depth > 999 || 		\
+	o->thickness < 0 || o->depth < 0 || o->depth > 999 ||		\
 	o->fill_style < UNFILLED ||					\
-	o->fill_style > NUMSHADES + NUMTINTS + NUMPATTERNS ||		\
+	o->fill_style >= NUMSHADES + NUMTINTS + NUMPATTERNS ||		\
 	o->style_val < 0.0
 
 typedef struct f_ellipse {
@@ -90,8 +93,9 @@ typedef struct f_ellipse {
 
 #define INVALID_ELLIPSE(e)	\
 	e->type < T_ELLIPSE_BY_RAD || e->type > T_CIRCLE_BY_DIA ||	\
-	COMMON_PROPERTIES(e) || e->direction != 1 && e->direction != 0 || \
-	e->radiuses.x < 0 || e->radiuses.y < 0
+	COMMON_PROPERTIES(e) || (e->direction != 1 && e->direction != 0) || \
+	e->radiuses.x == 0 || e->radiuses.y == 0 || \
+	e->angle < -7. || e->angle > 7.
 
 typedef struct f_arc {
 	int			type;
@@ -120,6 +124,8 @@ typedef struct f_arc {
 #define INVALID_ARC(a)	\
 	a->type < T_OPEN_ARC || a->type > T_PIE_WEDGE_ARC ||		\
 	COMMON_PROPERTIES(a) || a->cap_style < 0 || a->cap_style > 2 ||	\
+	a->center.x < COORD_MIN || a->center.x > COORD_MAX ||		\
+	a->center.y < COORD_MIN || a->center.y > COORD_MAX ||		\
 	(a->direction != 0 && a->direction != 1)
 
 typedef struct f_line {
@@ -158,11 +164,13 @@ typedef struct f_line {
 	(l->type == T_ARC_BOX && l->radius < 0)
 
 /* for colormap */
-
 #define		RED	0
 #define		GREEN	1
 #define		BLUE	2
-
+/* transparency: an index into the colormap: num_transp > 0, otherwise */
+#define		NO_TRANSPARENCY    0
+#define		TRANSP_COLOR	 (-1)	/* transparent color given as
+					   rgb triplet */
 typedef struct f_pic {
 	int			subtype;
 #define		P_EPS	0		/* EPS picture type */
@@ -177,12 +185,16 @@ typedef struct f_pic {
 	char			*file;
 	int			flipped;
 	unsigned char		*bitmap;
-#ifdef HAVE_X11_XPM_H
-	XpmImage		xpmimage;		/* for Xpm images */
-#endif
-	unsigned char		cmap[3][MAXCOLORMAPSIZE]; /* for color files */
+	unsigned char		cmap[3][MAXCOLORMAPSIZE];
 	int			numcols;	/* number of colors in cmap */
-	int			transp; /* transparent color (-1 if none) for GIFs */
+	/*
+	 * Png images can have a number of color map entries, or one color,
+	 * given as rgb triplet, that should be transparent. A full alpha
+	 * channel with partial transparency cannot be handled.
+	 */
+	int			num_transp;
+	unsigned char		transp_col[3];
+	unsigned char		*transp_cols;
 	float			hw_ratio;
 	struct f_pos		bit_size;
 #ifdef V4_0
@@ -238,9 +250,9 @@ typedef struct f_text {
 
 #define INVALID_TEXT(t)	\
 	t->type < T_LEFT_JUSTIFIED || t->type > T_RIGHT_JUSTIFIED ||	\
-	t->font < DEFAULT || t->font > MAX_PSFONT ||			\
+	t->font < DEFAULT || t->font > MAX_PSFONT || t->size < 0.0 ||	\
 	t->flags < DEFAULT || t->flags >= 2 * HIDDEN_TEXT ||		\
-	t->height < 0 || t->length < 0
+	t->height < 0 || t->length < 0 || t->angle < -7. || t->angle > 7.
 
 typedef struct f_control {
 	double			lx, ly, rx, ry;	/* used by older versions*/
@@ -339,3 +351,5 @@ typedef struct f_compound {
 
 #define		CLOSED_PATH		0
 #define		OPEN_PATH		1
+
+#endif /* OBJECT_H */
