@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2002 by Brian V. Smith
- * Parts Copyright (c) 2015 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,6 +16,9 @@
  *
  */
 
+#ifndef PI_H
+#define PI_H
+
 /* use the same PI as xfig uses, see xfig/src/fig.h */
 #undef M_PI
 #define M_PI	3.14159265358979323846
@@ -24,3 +27,5 @@
  * M_2PI is used in bound.c,
  * M_PI_2 is used in bound.c and genpict2e.c,
  * M_PI_4 is used in genpict2e.c */
+
+#endif /* PI_H */
