@@ -3,7 +3,7 @@
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
  * Parts Copyright (c) 1989-2015 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 2015-2019 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -16,7 +16,12 @@
  *
  */
 
+#ifndef READ_H
+#define READ_H
+
 extern int	read_fig(char *file_name, F_compound *obj);
 extern int	readfp_fig(FILE *fp, F_compound *obj);
 extern void	read_fail_message(char *file, int err);
 extern int	read_1_3_objects(FILE *fp, F_compound *obj);
+
+#endif /* READ_H */
