@@ -2,8 +2,8 @@
  * Fig2dev: Translate Fig code to various Devices
  * Copyright (c) 1991 by Micah Beck
  * Parts Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-2012 by Brian V. Smith
- * Parts Copyright (c) 2015-2018 by Thomas Loimer
+ * Parts Copyright (c) 1989-2015 by Brian V. Smith
+ * Parts Copyright (c) 2015-2020 by Thomas Loimer
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -26,14 +26,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_STRERROR
-#include <errno.h>
-#endif
 
-#include "fig2dev.h"
+#include "fig2dev.h"	/* includes object.h */
 #include "alloc.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
+//#include "object.h"
 #include "free.h"
+#include "messages.h"
 
 /*******    Fig 1.3 subtype of objects    *******/
 #define DRAW_ELLIPSE_BY_RAD	1
@@ -49,10 +47,7 @@
 #define DRAW_CLOSEDSPLINE	11
 #define DRAW_COMPOUND		13
 
-extern F_arrow		*forward_arrow(void), *backward_arrow(void);
 extern int		figure_modified;
-//extern int		line_no;
-extern int		num_object;
 
 static F_ellipse	*read_ellipseobject(FILE *fp);
 static F_line		*read_lineobject(FILE *fp);
@@ -60,6 +55,8 @@ static F_text		*read_textobject(FILE *fp);
 static F_spline		*read_splineobject(FILE *fp);
 static F_arc		*read_arcobject(FILE *fp);
 static F_compound	*read_compoundobject(FILE *fp);
+static F_arrow		*forward_arrow(void);
+static F_arrow		*backward_arrow(void);
 
 static char		Err_invalid[] = "Invalid %s object.";
 
@@ -92,6 +89,7 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 		put_msg("Invalid resolution information (%d).", pixperinch);
 		return -1;
 	}
+	ppi = pixperinch;
 	obj->nwcorner.x = pixperinch;
 	obj->nwcorner.y = coord_sys;
 	while (fscanf(fp, "%d", &object) == 1) {
@@ -102,7 +100,6 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 			ll = (ll->next = l);
 		    else
 			ll = obj->lines = l;
-		    num_object++;
 		    break;
 		case OBJ_SPLINE :
 		    if ((s = read_splineobject(fp)) == NULL) return(-1);
@@ -110,7 +107,6 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 			ls = (ls->next = s);
 		    else
 			ls = obj->splines = s;
-		    num_object++;
 		    break;
 		case OBJ_ELLIPSE :
 		    if ((e = read_ellipseobject(fp)) == NULL) return(-1);
@@ -118,7 +114,6 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 			le = (le->next = e);
 		    else
 			le = obj->ellipses = e;
-		    num_object++;
 		    break;
 		case OBJ_ARC :
 		    if ((a = read_arcobject(fp)) == NULL) return(-1);
@@ -126,7 +121,6 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 			la = (la->next = a);
 		    else
 			la = obj->arcs = a;
-		    num_object++;
 		    break;
 		case OBJ_TEXT :
 		    if ((t = read_textobject(fp)) == NULL) return(-1);
@@ -134,15 +128,13 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 			lt = (lt->next = t);
 		    else
 			lt = obj->texts = t;
-		    num_object++;
 		    break;
 		case OBJ_COMPOUND :
-		    if ((c = read_compoundobject(fp)) == NULL) return(-1);
+		    if ((c = read_compoundobject(fp)) == NULL) return(-3);
 		    if (lc)
 			lc = (lc->next = c);
 		    else
 			lc = obj->compounds = c;
-		    num_object++;
 		    break;
 		default:
 		    put_msg("Incorrect object code %d", object);
@@ -150,13 +142,9 @@ read_1_3_objects(FILE *fp, F_compound *obj)
 		} /*  switch */
 	    } /*  while */
 	if (feof(fp))
-	    return 0;
+		return 0;
 	else
-#ifdef HAVE_STRERROR
-	    return errno;
-#else
-	    return -1;
-#endif
+		return -3;
 }
 
 static F_arc *
@@ -228,13 +216,14 @@ read_compoundobject(FILE *fp)
 		&com->secorner.x, &com->secorner.y);
 	if (n != 4) {
 	    put_msg("Incorrect compound object format");
+	    free(com);
 	    return(NULL);
 	    }
 	while (fscanf(fp, "%d", &object) == 1) {
 	    switch (object) {
 		case OBJ_POLYLINE :
 		    if ((l = read_lineobject(fp)) == NULL) {
-			free_line(&l);
+			free_compound(&com);
 			return(NULL);
 			}
 		    if (ll)
@@ -244,7 +233,7 @@ read_compoundobject(FILE *fp)
 		    break;
 		case OBJ_SPLINE :
 		    if ((s = read_splineobject(fp)) == NULL) {
-			free_spline(&s);
+			free_compound(&com);
 			return(NULL);
 			}
 		    if (ls)
@@ -254,7 +243,7 @@ read_compoundobject(FILE *fp)
 		    break;
 		case OBJ_ELLIPSE :
 		    if ((e = read_ellipseobject(fp)) == NULL) {
-			free_ellipse(&e);
+			free_compound(&com);
 			return(NULL);
 			}
 		    if (le)
@@ -264,7 +253,7 @@ read_compoundobject(FILE *fp)
 		    break;
 		case OBJ_ARC :
 		    if ((a = read_arcobject(fp)) == NULL) {
-			free_arc(&a);
+			free_compound(&com);
 			return(NULL);
 			}
 		    if (la)
@@ -274,7 +263,7 @@ read_compoundobject(FILE *fp)
 		    break;
 		case OBJ_TEXT :
 		    if ((t = read_textobject(fp)) == NULL) {
-			free_text(&t);
+			free_compound(&com);
 			return(NULL);
 			}
 		    if (lt)
@@ -284,7 +273,7 @@ read_compoundobject(FILE *fp)
 		    break;
 		case OBJ_COMPOUND :
 		    if ((c = read_compoundobject(fp)) == NULL) {
-			free_compound(&c);
+			free_compound(&com);
 			return(NULL);
 			}
 		    if (lc)
@@ -296,17 +285,14 @@ read_compoundobject(FILE *fp)
 		    return(com);
 		} /*  switch */
 	    }
-	if (feof(fp))
-	    return(com);
-	else {
-#ifdef HAVE_STRERROR
-	    put_msg("Format error: %s", strerror(errno));
-#else
-	    put_msg("Format error.");
-#endif
-	    return(NULL);
-	    }
+	if (feof(fp)) {
+		return com;
+	} else {
+		put_msg("Fig 1.3 format error.");
+		free_compound(&com);
+		return NULL;
 	}
+}
 
 static F_ellipse *
 read_ellipseobject(FILE *fp)
@@ -369,6 +355,7 @@ read_lineobject(FILE *fp)
 	l->next = NULL;
 	l->points = Point_malloc(p);
 	l->points->next = NULL;
+	l->pic = NULL;
 	l->comments = NULL;
 	n = fscanf(fp, " %d %d %d %lf %d %d %d %d %d %d", &t,
 			&l->style, &l->thickness, &l->style_val,
@@ -468,7 +455,7 @@ read_splineobject(FILE *fp)
 		if (fscanf(fp, " %d %d", &x, &y) != 2) {
 			put_msg("incomplete spline object");
 			free_splinestorage(s);
-			return(NULL);
+			return NULL;
 		};
 		if (x == 9999) break;
 		Point_malloc(q);
@@ -478,6 +465,11 @@ read_splineobject(FILE *fp)
 		p->next = q;
 		p = q;
 	}
+	if (s->points->next == NULL) {
+		put_msg("spline with a single point");
+		free_splinestorage(s);
+		return NULL;
+	}
 	return(s);
 }
 
@@ -521,3 +513,37 @@ read_textobject(FILE *fp)
 		t->size = 18;
 	return t;
 }
+
+static F_arrow *
+forward_arrow(void)
+{
+	F_arrow		*a;
+
+	if (NULL == (Arrow_malloc(a))) {
+		put_msg(Err_mem);
+		return(NULL);
+	}
+	a->type = 0;
+	a->style = 0;
+	a->thickness = THICK_SCALE;
+	a->wid = 4.0;
+	a->ht = 8.0;
+	return a;
+}
+
+static F_arrow *
+backward_arrow(void)
+{
+	F_arrow		*a;
+
+	if (NULL == (Arrow_malloc(a))) {
+		put_msg(Err_mem);
+		return(NULL);
+	}
+	a->type = 0;
+	a->style = 0;
+	a->thickness = THICK_SCALE;
+	a->wid = 4.0;
+	a->ht = 8.0;
+	return a;
+}
