@@ -22,10 +22,10 @@
 #include <string.h>
 #include <math.h>
 
-#include "fig2dev.h"	/* includes "bool.h" */
-#include "alloc.h"
-#include "object.h"	/* does #include <X11/xpm.h> */
+#include "fig2dev.h"	/* includes bool.h and object.h*/
+//#include "object.h"
 #include "free.h"
+#include "messages.h"
 #include "trans_spline.h"
 
 
@@ -226,6 +226,13 @@ compute_closed_spline(F_spline *spline, float precision)
   if (!init_point_array(300, 200))
       return NULL;
 
+  if (!(spline->points /* p0 */ && spline->controls /* s0 */ &&
+	spline->points->next /* p1 */ && spline->controls->next /* s1 */ &&
+	spline->points->next->next && spline->controls->next->next/* p2, s2 */)
+		  ) {
+      fprintf(stderr, "A closed spline with less than three points.");
+      return NULL;
+  }
   INIT_CONTROL_POINTS(spline, p0, s0, p1, s1, p2, s2, p3, s3);
   COPY_CONTROL_POINT(first, s_first, p0, s0);
 
@@ -645,6 +652,6 @@ create_point(void)
     F_point	   *p;
 
     if ((p = (F_point *) malloc(POINT_SIZE)) == NULL)
-	put_msg(Err_mem);
+	fputs(Err_mem, stderr);
     return p;
 }
